clear all
set mem 32m
set more off

local 0 , `0'

syntax [, data irt mimic dif start ]

if "`data'" == "data" {
   tempfile demo pfq bpq bmx
   !wget ftp://ftp.cdc.gov/pub/Health_Statistics/NCHS/nhanes/2007-2008/DEMO_E.xpt
   !wget ftp://ftp.cdc.gov/pub/Health_Statistics/NCHS/nhanes/2007-2008/BMX_E.xpt
   !wget ftp://ftp.cdc.gov/pub/Health_Statistics/NCHS/nhanes/2007-2008/PFQ_E.xpt
   !wget ftp://ftp.cdc.gov/pub/Health_Statistics/NCHS/nhanes/2007-2008/BPQ_E.xpt

   foreach set of demo bmx pfq bpq {
      fdause `set'_e
      sort seqn
      save nhanes-`set', replace
   }

   use nhanes-pfq, clear
   keep seqn pfq061*

   merge seqn using nhanes-demo, keepusing( wtint2yr sdmvpsu sdmvstra ridageyr riagendr ridreth1 )
   drop _merge

   merge seqn using nhanes-bmx, keepusing( bmxbmi )
   drop _merge

   merge seqn using nhanes-bpq, keepusing( bpq020 )
   drop _merge

   save nhanes-adl-irt-bmx, replace

}

use nhanes-adl-irt-bmx

sum wtint
local meanw = r(mean)

drop pfq061a pfq061k pfq061t
recode pfq* bpq* bmx* (5 =.a) (7=.b) (9=.c)

gen byte highbp = (bpq020==1) & !mi( bpq020 )
gen byte female = riagendr - 1

egen _pfqmiss = rowmiss( pfq* )
drop if _pfqmiss > 10

local k=0
foreach x of varlist pfq061* {
   local ++k
   gen byte bpfq`k' = (`x'>1) if !missing( `x' )
   local pfqlab `pfqlab' `k' "`x'"
   ren `x' pfq`k'
}

drop pfq0*

if "`start'" == "start" {
  // initial estimates of the loadings
  confa (health: pfq*) [pw=wtint2], from(iv) search(off) vce( sbentler )
  predict health_cfa, reg

  estimates store cfa
  estimates save cfa, replace

  mat cfa = e(b)

  lab var health "PF score, CFA model"

  hist health [fw=int(wtint2)]
  graph export health-cfa.eps, replace

  scatter health age [fw=int(wtin2)], jitter(0.5) m(Oh) msize(0.35) ///
      || lowess health age , l1title("PF score, CFA model")

  graph export health-cfa-age.eps, replace
}

reshape long bpfq pfq, i(seqn) j(item)

keep if pfq < 5

gen wqt1 = 1
gen wgt2 = wtint2yr/`meanw'

lab def pfqlab `pfqlab'
lab val item pfqlab

tab item, gen( q )

unab allbutfirst : q2-q17
unab allitems : q1-q17

eq item1 : `allbutfirst'
eq items : `allitems'

if "`start'" == "start" {

// starting values
probit bpfq `allbutfirst'
mat oprob = e(b)

// set up the matrix of starting values
mat myoprob = J(1,1,.)
foreach x of varlist `allitems' {
   mat myoprob = (myoprob, _b[bpfq:`x'])
   local collabs `collabs' bpfq:`x'
}

* foreach x of varlist `allbutfirst' {
forvalues j = 2/17 {
   local x : word `j' of `items'
   mat myoprob = (myoprob, cfa[1,colnumb(cfa, "lambda_`j'_1:_cons")])
   local collabs `collabs' seq1_1l:`x'
}

mat myoprob = (myoprob, cfa[1,colnumb(cfa, "phi_1_1:_cons")])
local collabs `collabs' seq1_1:q1

mat myoprob = myoprob[1,2...]

mat colnames myoprob = `collabs'
mat rownames myoprob = y1

mat li myoprob

// start small
gllamm bpfq q* if seqn < 4200 , ///
   fam( bin ) link( probit ) nocons ///
   i( seqn ) eq( items ) weight( wgt ) ///
   trace dots iter(3) from( myoprob ) copy

est store from1
mat from1 = e(b)

mata : from1 = st_matrix("from1")
mata : mata matsave from1 from1, replace

file open labs using from1labs.txt, write text replace
file write labs `"`: colfullnames from1'"' _n
file close labs

}

if "`irt'" == "irt" {

mata : mata matuse from1, replace
mata : st_matrix("from1",from1)
file open labs using from1labs.txt, read text
file read labs flab
file close labs
mat colnames from1 = `flab'

gllamm bpfq q*, ///
   fam( bin ) link( probit ) i( seqn ) eq( items ) weight( wgt ) ///
   trace from( from1 ) copy nocons
est store irt
mat irt = e(b)

// predictions and graphs
gllapred health_irt, linpred

lab var health "PF score, IRT model"

hist health [fw=int(wtint2)]
graph export health-irt.eps, replace

scatter health age [fw=int(wtin2)] if item==10, jitter(0.5) m(Oh) msize(0.35) ///
    || lowess health age if item==10, l1title("PF score, IRT model")

graph export health-irt-age.eps, replace

}

if "`mimic'" == "mimic" {

mata : mata matuse from1, replace
mata : st_matrix("frommimic",from1)
file open labs using from1labs.txt, read text
file read labs fmlab
file close labs

// MIMIC
* mkspline age = ridageyr, knots( 0(20)80 ) cubic
mkspline age1 40 age2 60 age3 70 age4 = ridageyr
eq a1e : age1 age2 age3 age4 female highbp bmxbmi

di `"{inp}`fmlab'"'
foreach x of varlist age1-age4 female highbp bmxbmi {
   mat frommimic = (frommimic, 0)
   local fmlab `fmlab' a1e:`x'
}
di `"{inp}`fmlab'"'
mat colnames frommimic = `fmlab'

gllamm bpfq q*, ///
   fam( bin ) link( probit ) i( seqn ) eq( items ) weight( wgt ) geq( a1e ) ///
   trace from( frommimic ) skip nocons
est store mimic_bmi
est save mimic_bmi, replace
mat mimic = e(b)

gllapred health_mimic, linpred

lab var health "PF score, MIMIC model"

hist health [fw=int(wtint2)]
graph export health-mimic.eps, replace

scatter health age [fw=int(wtin2)] if item==10, jitter(0.5) m(Oh) msize(0.35) ///
    || lowess health age if item==10, l1title("PF score, MIMIC model")

graph export health-mimic-age.eps, replace

}

if "`dif'" == "dif" {
mata : mata matuse from1, replace
mata : st_matrix("fromdif",from1)
file open labs using from1labs.txt, read text
file read labs fdlab
file close labs

// DIF
cap noi xi, noomit: i.female*i.item
eq fitems  : `allitems' _IfemXite_1_*

unab finter : _IfemXite_1_*
foreach x of varlist `finter' {
   mat fromdif = (fromdif, 0, 0)
   local fdlab `fdlab' bpfq:`x' seq1_1l:`x'
}
mat colnames fromdif = `fdlab'

gllamm bpfq q* _IfemXite_1_* , ///
   fam( bin ) link( probit ) i( seqn ) eq( fitems ) weight( wgt ) ///
   trace from( fromdif ) skip nocons
est store dif
est save dif, replace
mat dif = e(b)

}

exit
