clear

tempfile accum

* demographic variables
fdause demo_e, clear
gen byte female = riagendr == 2 if !mi(riagendr)
lab var female "Female"

gen float age10 = ridageyr/10 if !mi(ridageyr)
gen float agesq = age10*age10/10 if !mi(ridageyr)
lab var age10 "Age/10"
lab var agesq "Age^2/1000"

gen byte black  = (ridreth==4) if !mi(ridreth)
gen byte hispanic = (ridreth<=2) if !mi(ridreth)
lab var black "Black"
lab var hispanic "Hispanic, Latino, Mexican American"

egen psu = group( sdm* )
lab var psu "Masked psu, sequentially numbered"

keep seqn sdm* wtint2yr wtmec2yr female age* black hispanic

sort seqn
save `accum'

* activity variables
fdause paq_e, clear
mvdecode paq610 paq625 paq640 paq655 paq670 , mv(99)
mvdecode pad615 pad630 pad645 pad660 pad675 , mv(9999)

gen float workvighrs = cond(paq605==1,paq610*pad615/60,0)  if paq605<7
lab var workvighrs "Vigorous activity at work, hours per week"

gen float workmodhrs = cond(paq620==1,paq625*pad630/60,0) if paq620<7
lab var workmodhrs "Moderate activity at work, hours per week"

gen float walkbikehrs = cond(paq635==1,paq640*pad645/60,0) if paq635<7
lab var walkbikehrs "Walking or biking for travel, hours per week"

gen float fitvighrs  = cond(paq650==1,paq655*pad660/60,0) if paq650<7
lab var fitvighrs "Vigorous fit activity, hours per week"

gen float fitmodhrs = cond(paq665==1,paq670*pad675/60,0) if paq665<7
lab var fitmodhrs "Moderate fit activity, hours per week"

keep seqn *hrs

foreach x of varlist *hrs {
   gen l`x' = log( `x'+1 )
   lab var l`x' "Log of `: var label `x''"
}

sort seqn
merge 1:1 seqn using `accum', keep(match)
drop _merge

save `accum', replace

* blood pressure
fdause bpx_e.xpt, clear

keep seqn bpxsy* bpxdi*
drop *4

foreach x of varlist bp* {
   replace `x' = `x'/10
   note `x' : Divided by 10
}

merge 1:1 seqn using `accum', keep(match)
drop _merge

lab data "NHANES data for SEM example"
compress
save nhanes-pa-bp, replace

exit
