

   mata : mata clear
   cap pro drop sem4gmm
   program define sem4gmm
      syntax varlist if [pw iw /], at(name) obsvar(varlist)
      // varlist is the list of vectorized residuals
      // at is the name of the parameter vector

      // create a variable to address the `if' condition
      marksample touse, novarlist

      // generate the implied matrix
      tempname Sigma
      mata : ParsToSigma( "`at'", "`Sigma'" )

      // produce vectorized deviations
      unab thevars : `obsvar'
	  tempvar wgt
	  if "`exp'" != "" qui gen double `wgt' = `exp'
	              else qui gen byte   `wgt' = 1
      mata : VechData("`thevars'", "`varlist'", 1, "`touse'", "`wgt'", st_matrix("`Sigma'") )
   end

   mata:

   void VechData( string scalar thevars, string scalar vechlist, | ///
      real scalar dev, string scalar tousename, string scalar weightname, real matrix Sigma) {

      real scalar p, i;
      real matrix x, m, v, w;

      // number of observed variables
      p = length( tokens( thevars ) )

      // center the variables before vech?
      if (missing(dev)) dev = 0;

      // subtract the supplied matrix?
      if (rows(Sigma)==0 | rows(Sigma)!=p ) Sigma = J( p, p, 0);

      // retrieve the data
      st_view(x=.,.,tokens(thevars), tousename )
	  st_view(w=.,.,weightname, tousename)
      m = mean(x,w) :*dev :/ rows(x)

      // retrieve the variables for vectorization
      st_view(v=.,.,tokens(vechlist), tousename )

      // compute vectorization observation by observation
      for( i=1;i<=rows(x); i++) {
         v[i,] = vech( (x[i,]-m)'*(x[i,]-m ) )' - vech(Sigma)'
      }

   }

   void ParsToSigma(string scalar atName, string scalar SigmaName) {

      real vector at, VarZeta;
      real matrix LambdaX, LambdaY, Phi, GammaZ, GammaKsi, PsiX, PsiY,
                  CovXX, CovYX, CovYY, CovZZ, CovXZ, CovYZ;

      // obtain the vector of parameters from Stata vector `at'
      at = st_matrix( atName )

      // check the number of parameters
      if (cols(at)!=45) {
         st_matrix( SigmaName, J(0,0,.) )
         return
      }

      // set up the parameter matrices
      // loadings:
      LambdaX = (1, at[1] , J(1,2,0) \ J(1,2,0), 1, at[2] )'
      LambdaY = (1, at[3] )'
      // order: workmod, workvig, walkbike, fitmod, fitvig

      // covariances of the exogenous variables
      Phi = invvech( at[4..31]' )

      // variance of the latent regression term
      VarZeta = at[32]

      // regression coefficients
      GammaZ = at[33..37]
      GammaKsi = at[38..39]

      // measurement errors
      PsiX  = diag( at[40..43] )
      PsiY  = diag( at[44..45] )

      // exogenous observed variables
      CovZZ = Phi[|(1,1)\(5,5)|]

      // indicators of exogenous latent variables
      CovXX = LambdaX * Phi[|(6,6)\(7,7)|] * LambdaX' + PsiX
      CovXZ = LambdaX * Phi[|(6,1)\(7,5)|]

      // indicators of endogenous latent variables
      CovYZ = LambdaY * ( GammaZ * Phi[|(1,1)\(5,5)|] + GammaKsi * Phi[|(6,1)\(7,5)|] )
      CovYX = LambdaY * ( GammaZ * Phi[|(1,6)\(5,7)|] + GammaKsi * Phi[|(6,6)\(7,7)|] ) * LambdaX'
      CovYY = LambdaY * ( GammaZ   * Phi[|(1,1)\(5,5)|] * GammaZ' +
                           GammaKsi * Phi[|(6,1)\(7,5)|] * GammaZ' +
                           GammaZ   * Phi[|(1,6)\(5,7)|] * GammaKsi' +
                           GammaKsi * Phi[|(6,6)\(7,7)|] * GammaKsi' +
                           VarZeta ) * LambdaY' + PsiY


      // return the matrix back to Stata
      st_matrix(SigmaName, (CovZZ, CovXZ', CovYZ' \ CovXZ, CovXX, CovYX' \ CovYZ, CovYX, CovYY) )

      // output a dot
      printf("{inp}.")
   }

   end
   // of mata piece

   exit
   